﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input.Touch;
using System;
using System.Collections.Generic;

namespace CloudFlick
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CloudGame : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World

        #region Clouds and Sprites

        interface ISprite
        {
            void Draw(CloudGame game);
            void Update(CloudGame game);
        }

        class Cloud : CloudGame.ISprite
        {
            public Texture2D CloudTexture;
            public Color[] CloudColors;
            public Vector2 CloudPosition;
            public Vector2 CloudSpeed;
            public SoundEffect CloudPopSound;

            public void Draw(CloudGame game)
            {
                game.spriteBatch.Draw(CloudTexture, CloudPosition, Color.White);
            }

            public void Update(CloudGame game)
            {
                CloudPosition += CloudSpeed;

                float widthAdjust = CloudTexture.Width + game.GraphicsDevice.Viewport.Width;
                float heightAdjust = CloudTexture.Height + game.GraphicsDevice.Viewport.Height;

                if (CloudPosition.X > game.GraphicsDevice.Viewport.Width)
                    CloudPosition.X -= widthAdjust;

                if (CloudPosition.X + CloudTexture.Width < 0)
                    CloudPosition.X += widthAdjust;

                if (CloudPosition.Y > game.GraphicsDevice.Viewport.Height)
                    CloudPosition.Y -= heightAdjust;

                if (CloudPosition.Y + CloudTexture.Height < 0)
                    CloudPosition.Y += heightAdjust;

                foreach (Plant p in game.plants)
                {
                    if (p.Dead) continue;

                    Vector2 plantCenter = p.PlantPosition + new Vector2(p.PlantTexture.Width / 2f, p.PlantTexture.Height / 2f);

                    if (CloudContains(plantCenter))
                    {
                        // Watering a plant

                        CloudPopSound.Play();
                        p.PlantAge = 1;

                        // Put the cloud back in the middle
                        CloudPosition.Y = game.rand.Next(game.GraphicsDevice.Viewport.Height - CloudTexture.Height);
                        CloudPosition.X = game.rand.Next(game.GraphicsDevice.Viewport.Width - CloudTexture.Width);

                        // Stop the cloud from moving
                        CloudSpeed = new Vector2(0, 0);
                    }
                }
            }

            public bool CloudContains(Vector2 pos)
            {
                // First do bounding box collision

                if (pos.X < CloudPosition.X) return false;
                if (pos.X >= (CloudPosition.X + CloudTexture.Width)) return false;
                if (pos.Y < CloudPosition.Y) return false;
                if (pos.Y >= (CloudPosition.Y + CloudTexture.Height)) return false;

                // Bounding boxes intersect. See if the position is a visible pixel

                int x = (int)(pos.X - CloudPosition.X);
                int y = (int)(pos.Y - CloudPosition.Y);

                if (CloudColors[y * CloudTexture.Width + x].A == 0)
                    return false;

                return true;
            }

            public Cloud(Texture2D inTexture, Vector2 inPosition, Vector2 inSpeed, SoundEffect inPopSound)
            {
                CloudTexture = inTexture;
                CloudColors = new Color[CloudTexture.Width * CloudTexture.Height];
                CloudTexture.GetData(CloudColors);
                CloudPosition = inPosition;
                CloudSpeed = inSpeed;
                CloudPopSound = inPopSound;
            }
        }

        class Plant : ISprite
        {
            public Texture2D PlantTexture;
            public Vector2 PlantPosition;

            public float PlantAge = 1.0f;
            public float PlantAgeStep;
            public bool Dead = false;

            public void Draw(CloudGame game)
            {
                if (!Dead)
                {
                    game.spriteBatch.Draw(PlantTexture, PlantPosition,
                        Color.White * PlantAge);
                }
            }

            public void Update(CloudGame game)
            {
                if (Dead) return;

                PlantAge -= PlantAgeStep;

                if (PlantAge < PlantAgeStep) Dead = true;
            }

            public Plant(Texture2D inTexture, Vector2 inPosition, float inAgeStep)
            {
                PlantTexture = inTexture;
                PlantPosition = inPosition;
                PlantAgeStep = inAgeStep;
            }
        }

        #endregion

        Random rand = new Random();

        int noOfPlants = 10;

        List<ISprite> gameSprites = new List<ISprite>();

        List<Plant> plants = new List<Plant>();

        Cloud waterCloud;



        public CloudGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            TouchPanel.EnabledGestures = GestureType.Flick;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Texture2D plantTexture = Content.Load<Texture2D>("Plant");

            for (int i = 0; i < noOfPlants; i++)
            {
                Vector2 plantPos =
                    new Vector2(rand.Next(GraphicsDevice.Viewport.Width),
                                rand.Next(GraphicsDevice.Viewport.Height));

                float ageSpeed = (rand.Next(100) + 10) / 70000f;

                Plant p = new Plant(plantTexture, plantPos, ageSpeed);

                gameSprites.Add(p);
                plants.Add(p);
            }


            Texture2D cloudTexture = Content.Load<Texture2D>("Cloud");
            SoundEffect cloudPopSound = Content.Load<SoundEffect>("Pop");

            Vector2 cloudPos =
                new Vector2(rand.Next(GraphicsDevice.Viewport.Width - cloudTexture.Width),
                            rand.Next(GraphicsDevice.Viewport.Height - cloudTexture.Height));

            Vector2 speed = new Vector2(0, 0);

            waterCloud = new Cloud(cloudTexture, cloudPos, speed, cloudPopSound);

            gameSprites.Add(waterCloud);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            while (TouchPanel.IsGestureAvailable)
            {
                GestureSample gesture = TouchPanel.ReadGesture();

                if (gesture.GestureType == GestureType.Flick)
                {
                    waterCloud.CloudSpeed = gesture.Delta / 100f;
                }
            }

            foreach (ISprite sprite in gameSprites)
                sprite.Update(this);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DarkGreen);

            spriteBatch.Begin();

            foreach (ISprite sprite in gameSprites)
                sprite.Draw(this);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
