﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using Windows.Devices.Sensors;

namespace CloudTipping
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CloudGame : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World

        #region Clouds, Plants and Sprites

        interface ISprite
        {
            void Draw(CloudGame game);
            void Update(CloudGame game);
        }

        class Cloud : CloudGame.ISprite
        {
            public Texture2D CloudTexture;
            public Color[] CloudColors;

            public Vector2 CloudPosition;
            public Vector2 CloudSpeed;
            public SoundEffect CloudPopSound;

            public void Draw(CloudGame game)
            {
                game.spriteBatch.Draw(CloudTexture, CloudPosition, Color.White);
            }

            public void Update(CloudGame game)
            {
                Vector3 acceleration = game.getAccReading();

                CloudSpeed.X += acceleration.X * 0.1f;

                CloudPosition += CloudSpeed;

                // Put objects back on the screen if they go off the edge

                float widthAdjust = CloudTexture.Width + game.GraphicsDevice.Viewport.Width;
                float heightAdjust = CloudTexture.Height + game.GraphicsDevice.Viewport.Height;

                if (CloudPosition.X > game.GraphicsDevice.Viewport.Width)
                    CloudPosition.X -= widthAdjust;

                if (CloudPosition.X + CloudTexture.Width < 0)
                    CloudPosition.X += widthAdjust;

                if (CloudPosition.Y > game.GraphicsDevice.Viewport.Height)
                    CloudPosition.Y -= heightAdjust;

                if (CloudPosition.Y + CloudTexture.Height < 0)
                    CloudPosition.Y += heightAdjust;

                // See if we have gone over a plant

                foreach (Plant p in game.plants)
                {
                    if (p.Dead) continue;

                    Vector2 plantCenter = p.PlantPosition + new Vector2(p.PlantTexture.Width / 2f, p.PlantTexture.Height / 2f);

                    if (CloudContains(plantCenter))
                    {
                        CloudPopSound.Play();
                        p.PlantAge = 1;

                        CloudPosition.Y = -CloudTexture.Height;
                        CloudPosition.X = game.rand.Next(0, game.GraphicsDevice.Viewport.Width);
                        CloudSpeed = new Vector2(CloudSpeed.X, (game.rand.Next(0, 200) + 50) / 100f);
                    }
                }
            }

            public bool CloudContains(Vector2 pos)
            {
                // First do bounding box collision

                if (pos.X < CloudPosition.X) return false;
                if (pos.X >= (CloudPosition.X + CloudTexture.Width)) return false;
                if (pos.Y < CloudPosition.Y) return false;
                if (pos.Y >= (CloudPosition.Y + CloudTexture.Height)) return false;

                // Bounding boxes intersect. See if the position is a visible pixel

                int x = (int)(pos.X - CloudPosition.X);
                int y = (int)(pos.Y - CloudPosition.Y);

                if (CloudColors[y * CloudTexture.Width + x].A == 0)
                    return false;

                return true;
            }

            public bool CloudIntersects(Cloud c)
            {
                if (CloudPosition.X + CloudTexture.Width < c.CloudPosition.X) return false;
                if (c.CloudPosition.X + c.CloudTexture.Width < CloudPosition.X) return false;
                if (CloudPosition.Y + CloudTexture.Height < c.CloudPosition.Y) return false;
                if (c.CloudPosition.Y + c.CloudTexture.Height < CloudPosition.Y) return false;
                return true;
            }

            public Cloud(Texture2D inTexture, Vector2 inPosition, Vector2 inSpeed, SoundEffect inPopSound)
            {
                CloudTexture = inTexture;
                CloudColors = new Color[CloudTexture.Width * CloudTexture.Height];
                CloudTexture.GetData(CloudColors);
                CloudPosition = inPosition;
                CloudSpeed = inSpeed;
                CloudPopSound = inPopSound;
            }
        }

        class Plant : ISprite
        {
            public Texture2D PlantTexture;
            public Vector2 PlantPosition;

            public float PlantAge = 1.0f;
            public float PlantAgeStep;
            public bool Dead = false;

            public void Draw(CloudGame game)
            {
                if (!Dead)
                {
                    game.spriteBatch.Draw(PlantTexture, PlantPosition, Color.White * PlantAge);
                }
            }

            public void Update(CloudGame game)
            {
                if (Dead) return;

                PlantAge -= PlantAgeStep;

                if (PlantAge < PlantAgeStep) Dead = true;
            }

            public Plant(Texture2D inTexture, Vector2 inPosition, float inAgeStep)
            {
                PlantTexture = inTexture;
                PlantPosition = inPosition;
                PlantAgeStep = inAgeStep;
            }
        }

        List<ISprite> gameSprites = new List<ISprite>();
        List<Plant> plants = new List<Plant>();

        #endregion

        #region Accelerometer

        Accelerometer accel;
        Vector3 accelReading;

        private void startAccelerometer()
        {
            accel = Accelerometer.GetDefault();

            if ( accel != null)
                accel.ReadingChanged += accel_ReadingChanged;
        }

        void accel_ReadingChanged(Accelerometer sender, AccelerometerReadingChangedEventArgs args)
        {
            lock (this)
            {
                accelReading.X = (float)args.Reading.AccelerationX;
                accelReading.Y = (float)args.Reading.AccelerationY;
                accelReading.Z = (float)args.Reading.AccelerationZ;
            }
        }

        Vector3 getAccReading()
        {
            Vector3 result;

            lock (this)
            {
                result = new Vector3(accelReading.X, accelReading.Y, accelReading.Z);
            }

            return result;
        }


        #endregion

        Random rand = new Random();

        int noOfClouds = 5;
        int noOfPlants = 10;

        public CloudGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            startAccelerometer();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Texture2D plantTexture = Content.Load<Texture2D>("Plant");

            for (int i = 0; i < noOfPlants; i++)
            {
                Vector2 position =
                    new Vector2(rand.Next(GraphicsDevice.Viewport.Width),
                                rand.Next(GraphicsDevice.Viewport.Height));

                float ageSpeed = (rand.Next(100) + 10) / 7000f;

                Plant p = new Plant(plantTexture, position, ageSpeed);

                gameSprites.Add(p);
                plants.Add(p);
            }

            Texture2D cloudTexture = Content.Load<Texture2D>("Cloud");
            SoundEffect cloudPopSound = Content.Load<SoundEffect>("Pop");

            for (int i = 0; i < noOfClouds; i++)
            {
                Vector2 position =
                    new Vector2(rand.Next(GraphicsDevice.Viewport.Width),
                                rand.Next(GraphicsDevice.Viewport.Height));

                Vector2 speed = new Vector2(0, (rand.Next(0, 200) + 50) / 100f);

                Cloud c = new Cloud(cloudTexture, position, speed, cloudPopSound);

                gameSprites.Add(c);
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            foreach (ISprite sprite in gameSprites)
                sprite.Update(this);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DarkGreen);

            spriteBatch.Begin();

            foreach (ISprite sprite in gameSprites)
                sprite.Draw(this);

            spriteBatch.End();


            base.Draw(gameTime);
        }
    }
}
